<div class="abc" style="float: left; margin-left: 15%; margin-right:15%; width: 70%" >
    <h2>Mapping     <i class="icon-help-circled" onClick="toggle1(this)" style="cursor:pointer;"></i></h2>
    <div class="box">
      <div class="description" style="display: none;">
        <p>If '--not_consensus' argument was used, Reads were used instead of UMIs.</p>
        <p><b>UMIs(or Reads) Mapped to Any VDJ Gene</b> : UMIs(or Reads) Mapped to any germline VDJ gene segments.</p>
        <p><b>UMIs(or Reads) with CDR3</b> : UMIs(or Reads) with CDR3 sequence.</p>
        <p><b>UMIs(or Reads) with Correct CDR3</b> : some UMIs(or Reads) with CDR3 might have stop codon and these UMIs(or Reads) are classified as incorrect.</p>
        <p><b>UMIs(or Reads) Mapped Confidently to VJ Gene</b> : UMIs(or Reads) mapped to VJ gene pairs and with correct CDR3.</p>
        <p><b>UMIs(or Reads) Mapped to IGH</b> : UMIs(or Reads) mapped confidently to IGH chain.</p>
        <p><b>UMIs(or Reads) Mapped to IGL</b> : UMIs(or Reads) mapped confidently to IGL chain.</p>
        <p><b>UMIs(or Reads) Mapped to IGK</b> : UMIs(or Reads) mapped confidently to IGK chain.</p>
    </div>
      <table style="float: left; margin-left: 0%; margin-right:3%; width: 47%">
        {% for item in BCR_mapping_vdj_summary %}
          {% if loop.index <= (loop.length+1)/2 %}
          <tr>
            {% for i in item %} 
            <td>{{ i|e }}</td>
            {% endfor %}
          </tr>
          {% endif %}
        {% endfor %}
      </table>

      <table style="float: left; margin-left: 3%; margin-right:0%; width: 47%">
        {% for item in BCR_mapping_vdj_summary %}
          {% if loop.index > (loop.length+1)/2 %}
          <tr>
            {% for i in item %} 
            <td>{{ i|e }}</td>
            {% endfor %}
          </tr>
          {% endif %}
        {% endfor %}
      </table>
      <div class="clear" ></div>
    </div>
  </div>