<div class="abc" style="float: left; margin-left: 15%; margin-right:15%; width: 70%" >
  <h2>Mapping    <i class="icon-help-circled" onClick="toggle1(this)" style="cursor:pointer;"></i></h2>

  <div class="box">
    
    <div class="description" style="display: none;">
      <p><b>Reads Mapped</b> : R2 reads that successfully mapped to linker and tag-barcode.</p>
      <p><b>Reads Unmapped too Short</b> : Unmapped R2 reads because read length < linker length + tag-barcode length.</p>
      <p><b>Reads Unmapped Invalid Linker</b> : Unmapped R2 reads because of too many mismatches in linker sequence.</p>
      <p><b>Reads Unmapped Invalid Barcode</b> : Unmapped R2 reads because of too many mismatches in tag-barcode sequence.</p>
    </div>

    <table style="float: left; margin-left: 0%; margin-right:3%; width: 47%">
      {% for item in mapping_tag_summary %}
        {% if loop.index <= (loop.length+1)/2 %}
        <tr>
          {% for i in item %} 
          <td>{{ i|e }}</td>
          {% endfor %}
        </tr>
        {% endif %}
      {% endfor %}
    </table>

    <table style="float: left; margin-left: 3%; margin-right:0%; width: 47%">
      {% for item in mapping_tag_summary %}
        {% if loop.index > (loop.length+1)/2  %}
        <tr>
          {% for i in item %} 
          <td>{{ i|e }}</td>
          {% endfor %}
        </tr>
        {% endif %}
      {% endfor %}
    </table>
    <div class="clear" ></div>
  </div>
</div>

