<div class="abc" style="float: left; margin-left: 15%; margin-right:15%; width: 70%" >
    <h2>Cell    <i class="icon-help-circled" onClick="toggle1(this)" style="cursor:pointer;"></i></h2>
    <div class="box">
      <div class="description" style="display: none;">
        <p><b>Estimated Number of cells</b> : number of barcodes considered as cell-associated.</p>
        <p><b>Median UMIs per Cell</b> : median number of UMI mapped to each chain per cell.</p>
        <p><b>Cell with Heavy and Light Chain</b> : cells with as least {{ iUMI }} UMI mapped to each chain.</p>
        <p><b>Cell with Barcode Match </b> : cells with barcode matched with scRNA-seq library.</p>
        <p><b>Cell with barcode Match, Heavy and Light chain</b> : cell with matched barcode and with as least {{ iUMI }} UMI mapped to each chain.</p>
    </div>
      <table style="float: left; margin-left: 0%; margin-right:3%; width: 47%">
        {% for item in BCR_count_vdj_summary %}
          <tr>
            {% for i in item %} 
            <td>{{ i|e }}</td>
            {% endfor %}
          </tr>
        {% endfor %}
      </table>

      </table>

      <div id="myDivUMI" style="float: left; margin-left: 3%; margin-right:0%; width: 47%">
        <script>
            var index1 = [];
            for (var i = 0; i < {{ CB_num }}; i++) {
                index1.push(i + 1)
            }

            var index2 = [];
            for (var i = 0; i < {{ UB_num }}; i++) {
                index2.push(i + 1 + {{ CB_num }})
            }

            var trace0 = {
                x: index1,
                y: {{ Cells }},
                mode: 'lines',
                name: 'Cells',
                line: {
                    color: '#337bb6',
                    width: 3
                }
            };
            var trace1 = {
                x: index2,
                y: {{ Background }},
                mode: 'lines',
                name: 'Background',
                line: {
                    color: '#D3D3D3',
                    width: 3
                }
            };
            var data = [ trace0, trace1 ];
            var layout = {
                height: 313,
                width: 470,
                margin: {
                      l: 50,
                      r: 0,
                      b: 30,
                      t: 30,
                },
                title: 'Cells', 
                xaxis: {
                    title: 'Barcodes',
                    type: 'log'
                },
                yaxis: {
                    title: 'UMI counts',
                    type: 'log'
                },
            }
          var config = { 
              modeBarButtonsToRemove: ["zoom2d", "pan2d", "select2d", "lasso2d", "zoomIn2d", "zoomOut2d", 
                                       "hoverCompareCartesian", "toggleSpikelines", "toImage", "autoScale2d" ], 
              displaylogo: false, 
              showTips: true ,
              responsive: true,}
            Plotly.newPlot('myDivUMI', data, layout, config);
        </script>       
      </div>

      <div class="clear" ></div>
    </div>
  </div>